<?php

namespace App\Http\Controllers;
use App\Models\Style;
use App\Models\StyleDetail;
use App\Models\Size;
use App\Models\Orientation;
use App\Traits\ImageUploadTrait;
use App\Models\LanguageTranslator;
use Illuminate\Http\Request;
use Validator;

class StylesController extends Controller
{
    use ImageUploadTrait;
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $styles= Style::all();
        return view('admin.styles.index',compact('styles'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $orientations = Orientation::select('id',"orientation")->with("sizes")->get();
        $orientationSizes = [];
        foreach($orientations as $key=>$orientation){
            $orientationSizes[$orientation->orientation] = $orientation->sizes()->select("id","title")->get();
        }
        $orientationSizes = json_encode($orientationSizes);
        return view('admin.styles.create',compact('orientations','orientationSizes'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        return $this->uploadImage('left_frame','uploads/styles/leftFrames',$request,'',true);
        $validations = Validator::make($request->all(),$this->rules($request),$this->messages($request));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        $style = new Style();
        $style->title = $request->title;
        $style->icon = $this->uploadImage('icon','uploads/styles/icons',$request,'',false);
        $style->image = $this->uploadImage('image','uploads/styles/images',$request,'',false);
        $style->left_frame = $this->uploadImage('left_frame','uploads/styles/leftFrames',$request,'',true);
        if($style->save()){
            foreach($request->orientations as $key=>$value){
                $styleDetail = new StyleDetail();
                $styleDetail->style_id = $style->id;
                $styleDetail->orientation_id = $request->orientations[$key];
                $styleDetail->size_id = $request->sizes[$key];
                $styleDetail->price = $request->prices[$key];
                $styleDetail->save();
            }

            return response()->json(['success' => true, 'message' => 'Style has been added successfully']);
        }

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $style = Style::find($id);
        return view('admin.styles.show',compact('style'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $style = Style::find($id);
        $orientations = Orientation::select('id',"orientation")->with("sizes")->get();
        $orientationSizes = [];
        foreach($orientations as $key=>$orientation){
            $orientationSizes[$orientation->orientation] = $orientation->sizes()->select("id","title")->get();
        }
        $orientationSizes = json_encode($orientationSizes);
        return view('admin.styles.edit',compact('style','orientationSizes','orientations'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $style = Style::find($id);
        $totalEntries = count($style->details()->pluck('id')->toArray());
        $arrDiff = $request->old_ids ? array_values(array_diff($style->details->pluck('id')->toArray(),array_map('intval', array_values($request->old_ids)))):$style->details->pluck('id')->toArray();
        $validations = Validator::make($request->all(),$this->rules($request,true),$this->messages($request));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        $style->title = $request->title;
        if($request->hasFile("icon")){ $style->icon = $this->uploadImage('icon','uploads/styles/icons',$request,'',false); }
        if($request->hasFile("image")){ $style->image = $this->uploadImage('image','uploads/styles/images',$request,'',false); }
        if($request->hasFile("left_frame")){ $style->left_frame = $this->uploadImage('left_frame','uploads/styles/leftFrames',$request,'',true); }
        
        
        if($style->save()){

            foreach($request->orientations as $key=>$value){
                $styleDetail = isset($request->ids[$key]) && $request->ids[$key]!='0'? StyleDetail::find($request->ids[$key]) : new StyleDetail();
                $styleDetail->style_id = $style->id;
                $styleDetail->orientation_id = $request->orientations[$key];
                $styleDetail->size_id = $request->sizes[$key];
                $styleDetail->price = $request->prices[$key];
                $styleDetail->save();
            }

            StyleDetail::whereIn("id",$arrDiff)->delete();

            return response()->json(['success' => true, 'message' => 'Style has been updated successfully']);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        if(Style::where('id',$id)->delete()){
            return response()->json(['success' => true, 'message' =>'Style has been deleted successfully']);
        }
    }

    // get rules for create and update
    private function rules($request,$update=false)
    {
        $rules = [
            "title"  => ['required'],
        ];

        if(!$update){
            $rules["icon"]  = ['required'];
            $rules["image"]  = ['required'];
            $rules["left_frame"]  = ['required'];
        }

        $rules["orientations"]  = ['required'];
        $rules["sizes"]  = ['required'];
        $rules["prices"]  = ['required'];
        $rules["orientations.*"]  = ['required'];
        $rules["sizes.*"]  = ['required'];
        $rules["prices.*"]  = ['required'];
        return $rules;
    }

    // error messages for validation
    private function messages($request)
    {
        $messages = [];
        foreach($request->all() as $key=>$value){
            if(is_array($value)){
                foreach ($value as $k => $v) {
                    $arr = explode('_',$key);
                    $first = ucfirst($arr[0]);
                    $second = (isset($arr[1])) ? str_replace('s','',$arr[1]):"";
                    $kkk = $k + 1;
                    $messages["$key.$k.required"] = " $first entry number $kkk $second field is required";
                }
            }
        }
        return $messages;
    }

    // get styles and their details
    public function stylesAndDetails(Request $request){
        // $basic = LanguageTranslator::whereIn('option',["style","orientation","size"])->where('language',$request->language)->get();
        $basic = LanguageTranslator::whereIn('option',["style","orientation","size","drop_text",'checkout'])->where('language',"Czech")->get();
        $basicArr = [];
        foreach($basic as $b){ $basicArr[$b['option']] = $b['value']; }
        $stylesTranslation = LanguageTranslator::select('option',"value")->where('language',$request->language)->where("option","like","style_%")->get();
        $orientationsTranslation = LanguageTranslator::select('option',"value")->where('language',$request->language)->where("option","like","orientation_%")->get();
        $sizesTranslations = LanguageTranslator::select('option',"value")->where('language',$request->language)->where("option","like","size_%")->get();
        $changeStylesArray =[];
        $changeOrientaionsArray =[];
        $changeSizesArray =[];
        foreach($stylesTranslation as $key=>$translation){ $changeStylesArray[str_replace("style_",'',$translation['option'])] = $translation['value']; }
        foreach($orientationsTranslation as $key=>$translation){ $changeOrientaionsArray[str_replace("orientation_",'',$translation['option'])] = $translation['value']; }
        foreach($sizesTranslations as $key=>$translation){ $changeSizesArray[str_replace("size_",'',$translation['option'])] = $translation['value']; }
        $styleIdsArr = array_keys($changeStylesArray);
        $orientationIdsArr = array_keys($changeOrientaionsArray);
        $sizeIdsArr = array_keys($changeSizesArray);
        // return [$changeStylesArray,$changeOrientaionsArray,$changeSizesArray];
        $styles = Style::whereIn('id',$styleIdsArr)->get();
        $stylesArr = [];
        foreach($styles as $style){
            $styleArr = [];
            $styleArr["style_id"]=$style->id;
            $styleArr["style_title"]=$changeStylesArray[$style->id];
            $styleArr["style_image"]=$style->image;
            $styleArr["style_icon"]=$style->icon;
            $styleArr["style_left_frame"]=$style->left_frame;
            if($style->details->count() >0){
                $detailArr = [];
                foreach($style->details()->whereIn("orientation_id",$orientationIdsArr)->distinct("orientation_id")->get() as $detail){
                    $orientationDetail = [];
                    $orientationDetail['orientation_id'] = $detail->orientation->id;
                    $orientationDetail['orientation_title'] = $changeOrientaionsArray[$detail->orientation->id];
                    $orientationDetail['orientation_icon'] = $detail->orientation->icon;
                    $orientationSizes = StyleDetail::where("style_id",$style->id)->where('orientation_id',$detail->orientation_id)->whereIn('size_id',$sizeIdsArr)->get();
                    $sizesArr = [];
                    if($orientationSizes->count() > 0){
                        foreach($orientationSizes as $orientationSize){
                            $sizeArr = [];
                            $sizeArr['id'] = $orientationSize->size->id;
                            $sizeArr['title'] =  $changeSizesArray[$orientationSize->size->id];
                            $sizeArr['icon'] = $orientationSize->size->icon;
                            $sizeArr['image'] = $orientationSize->size->image;
                            $sizeArr['width'] = $orientationSize->size->width;
                            $sizeArr['height'] = $orientationSize->size->height;
                            $sizeArr['price'] = $orientationSize->price;
                            $sizesArr[] = $sizeArr;
                        }

                    }

                    $orientationDetail["sizes"] = $sizesArr;
                    $detailArr[$detail->orientation_id] = $orientationDetail;
                }
                $styleArr['style_details'] = $detailArr;
            }else{
                $styleArr['style_details'] = [];
            }

            $stylesArr[$style->id] = $styleArr;
        }
        return response()->json(["basic"=>$basicArr,"styles"=>$stylesArr],200);
    }
}