<?php

namespace App\Http\Controllers;

use App\Models\LanguageTranslator;
use App\Models\Style;
use App\Models\Size;
use App\Models\Orientation;
use Illuminate\Http\Request;
use Validator;
class LanguageTranslatorController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $translators = LanguageTranslator::select("id",'option','value','language')->get()->groupBy("language");
        return view('admin.languagetranslators.index',compact('translators'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.languagetranslators.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validations = Validator::make($request->all(),$this->rules($request));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        foreach($request->except(["_token","language"]) as $name=>$value){
            if(LanguageTranslator::where("option",$name)->where('language',$request->language)->first()==null){
                $translator = new LanguageTranslator();
                $translator->language = $request->language;
                $translator->option = $name;
                $translator->value = $value;
                $translator->save();
            }
        }
        return response()->json(['success' => true, 'message' => 'Translator has been added successfully']);

    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\LanguageTranslator  $languageTranslator
     * @return \Illuminate\Http\Response
     */
    public function show(LanguageTranslator $languageTranslator)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\LanguageTranslator  $languageTranslator
     * @return \Illuminate\Http\Response
     */
    public function edit($language)
    {
        $response = getOldIds($language);
        $styles = Style::whereNotIn("id",$response['styles'])->get();
        $orientations = Orientation::whereNotIn("id",$response['orientations'])->get();
        $sizes = Size::whereNotIn("id",$response['sizes'])->get();
        $translators = LanguageTranslator::where("language",$language)->get();
        return view('admin.languagetranslators.edit',compact('translators','language','sizes','orientations','styles'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\LanguageTranslator  $languageTranslator
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $language)
    {
        $validations = Validator::make($request->all(),$this->rules($request));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        LanguageTranslator::where("language",$language)->delete();
        foreach($request->except(["_token","_method","language"]) as $name=>$value){
            if(LanguageTranslator::where("option",$name)->where('language',$request->language)->first()==null){
                $translator = new LanguageTranslator();
                $translator->language = $request->language;
                $translator->option = $name;
                $translator->value = $value;
                $translator->save();
            }
        }
        return response()->json(['success' => true, 'message' => 'Translator has been updated successfully']);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\LanguageTranslator  $languageTranslator
     * @return \Illuminate\Http\Response
     */
    public function destroy(LanguageTranslator $languageTranslator)
    {
        //
    }

    // get rules for create and update
    private function rules($request,$update=false)
    {
        $rules = ["language"=>['required']];
        foreach($request->except("_token") as $name=>$value){
            $rules[$name] = ['required'];
        }
        return $rules;
    }
}
