<?php

namespace App\Http\Controllers;

use App\Models\Orientation;
use App\Traits\ImageUploadTrait;
use Illuminate\Http\Request;
use Validator;

class OrientationController extends Controller
{
    use ImageUploadTrait;
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $orientations= Orientation::all();
        return view('admin.orientations.index',compact('orientations'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('admin.orientations.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validations = Validator::make($request->all(),$this->rules($request));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        if(Orientation::where("orientation",$request->orientation)->first()!=null){return response()->json(['success' => false, 'message' =>['orientation'=>["Sorry! ".$request->orientation." orientation is already exist"]]]);}
        $orientation = new Orientation();
        $orientation->orientation = $request->orientation;
        $orientation->icon = $this->uploadImage('icon','uploads/orientations/icons',$request,'',false);
        if($orientation->save()){
            return response()->json(['success' => true, 'message' => 'Orientation has been added successfully']);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Orientation  $orientation
     * @return \Illuminate\Http\Response
     */
    public function show(Orientation $orientation)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Orientation  $orientation
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $orientation = Orientation::find($id);
        return view('admin.orientations.edit',compact('orientation'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Orientation  $orientation
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request,$id)
    {
        $orientation = Orientation::find($id);
        $validations = Validator::make($request->all(),$this->rules($request,true));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        $orientation->orientation = $request->orientation;
        if($request->hasFile("icon")){ $orientation->icon = $this->uploadImage('icon','uploads/orientations/icons',$request,'',false); }
        if($orientation->save()){
            return response()->json(['success' => true, 'message' => 'Orientation has been added successfully']);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Orientation  $orientation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Orientation $orientation)
    {
        if($orientation->delete()){
            return response()->json(['success' => true, 'message' =>'Orientation has been deleted successfully']);
        }
    }

    // get rules for create and update
    private function rules($request ,$update=false)
    {
        $rules = ["orientation"  => ['required'] ];

        if(!$update){
            $rules["icon"]  = ['required'];
        }
        return $rules;
    }

}
