<?php

namespace App\Http\Controllers;

use App\Models\Size;
use App\Models\Orientation;
use Illuminate\Http\Request;
use Validator;
use App\Traits\ImageUploadTrait;

class SizeController extends Controller
{
    use ImageUploadTrait;
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $sizes= Size::all();
        return view('admin.sizes.index',compact('sizes'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $orientations = Orientation::all();
        return view('admin.sizes.create',compact('orientations'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validations = Validator::make($request->all(),$this->rules($request));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        $size = new Size();
        $size->orientation_id = $request->orientation_id;
        $size->title = $request->title;
        $size->width = $request->width;
        $size->height = $request->height;
        $size->mobile_width = $request->mobile_width;
        $size->mobile_height = $request->mobile_height;
        $size->icon = $this->uploadImage('icon','uploads/sizes/icons',$request,'',false);
        $size->image = $this->uploadImage('image','uploads/sizes/images',$request,'',false);
        if($size->save()){
            return response()->json(['success' => true, 'message' => 'Size has been added successfully']);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Size  $size
     * @return \Illuminate\Http\Response
     */
    public function show(Size $size)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Size  $size
     * @return \Illuminate\Http\Response
     */
    public function edit(Size $size)
    {
        $orientations = Orientation::all();
        return view('admin.sizes.edit',compact('size','orientations'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Size  $size
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Size $size)
    {
        $validations = Validator::make($request->all(),$this->rules($request,true));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        $size->orientation_id = $request->orientation_id;
        $size->title = $request->title;
        $size->width = $request->width;
        $size->height = $request->height;
        $size->mobile_width = $request->mobile_width;
        $size->mobile_height = $request->mobile_height;
        if($request->hasFile("icon")){ $size->icon = $this->uploadImage('icon','uploads/sizes/icons',$request,'',false); }
        if($request->hasFile("image")){ $size->image = $this->uploadImage('image','uploads/sizes/images',$request,'',false); }
        if($size->save()){
            return response()->json(['success' => true, 'message' => 'Size has been added successfully']);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Size  $size
     * @return \Illuminate\Http\Response
     */
    public function destroy(Size $size)
    {
        if($size->delete()){
            return response()->json(['success' => true, 'message' =>'Size has been deleted successfully']);
        }
    }

    // get rules for create and update
    private function rules($request ,$update=false)
    {
        $rules = ["orientation_id"=>["required"],"title"  => ['required'] ];

        if(!$update){
            $rules["icon"]  = ['required'];
            $rules["image"]  = ['required'];
        }
        return $rules;
    }
}
