<?php

namespace App\Http\Controllers;
use App\Models\Style;
use App\Models\StyleDetail;
use App\Models\Size;
use App\Models\Orientation;
use App\Traits\ImageUploadTrait;
use Illuminate\Http\Request;
use Validator;

class StylesControllerBKP extends Controller
{
    use ImageUploadTrait;
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $styles= Style::all();
        return view('admin.styles.index',compact('styles'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $orientations = Orientation::all();
        $sizes = Size::all();
        return view('admin.styles.create',compact('orientations','sizes'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validations = Validator::make($request->all(),$this->rules($request),$this->messages($request));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        $style = new Style();
        $style->title = $request->title;
        $style->icon = $this->uploadImage('icon','uploads/styles/icons',$request,'');
        $style->image = $this->uploadImage('image','uploads/styles/images',$request,'');
        $style->left_frame = $this->uploadImage('left_frame','uploads/styles/leftFrames',$request,'');
        if($style->save()){
            if($request->vertical_check){
                foreach($request->vertical_prices as $key=>$value){
                    $styleDetail = new StyleDetail();
                    $styleDetail->style_id = $style->id;
                    $styleDetail->orientation = $request->vertical_check;
                    $styleDetail->price = $request->vertical_prices[$key];
                    $styleDetail->size = $request->vertical_sizes[$key];
                    $styleDetail->price = $request->vertical_prices[$key];
                    $styleDetail->image = $this->uploadFiles($request->file("vertical_images")[$key],"uploads/styles/details/images");
                    $styleDetail->save();
                }
            }

            if($request->horizontal_check){
                foreach($request->horizontal_prices as $key=>$value){
                    $styleDetail = new StyleDetail();
                    $styleDetail->style_id = $style->id;
                    $styleDetail->orientation = $request->horizontal_check;
                    $styleDetail->price = $request->horizontal_prices[$key];
                    $styleDetail->size = $request->horizontal_sizes[$key];
                    $styleDetail->price = $request->horizontal_prices[$key];
                    $styleDetail->image = $this->uploadFiles($request->file("horizontal_images")[$key],"uploads/styles/details/images");
                    $styleDetail->save();
                }
            }

            return response()->json(['success' => true, 'message' => 'Style has been added successfully']);
        }

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $style = Style::find($id);
        return view('admin.styles.show',compact('style'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $style = Style::find($id);
        $orientations = Orientation::all();
        $sizes = Size::all();
        $verticals = $style->details()->where("orientation","vertical")->get();
        $horizontals = $style->details()->where("orientation","horizontal")->get(); 
        return view('admin.styles.edit',compact('style','verticals','horizontals','orientations','sizes'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $style = Style::find($id);
        $totalEntries = count($style->details()->pluck('id')->toArray());
        $arrDiff = $request->old_ids ? array_values(array_diff($style->details->pluck('id')->toArray(),array_map('intval', array_values($request->old_ids)))):$style->details->pluck('id')->toArray();
        $validationscCheck = (count($arrDiff) > 0 && count($arrDiff)==$totalEntries)?true:false;
        $validations = Validator::make($request->all(),$this->rules($request,$validationscCheck,true),$this->messages($request));
        if ($validations->fails()) {return response()->json(['success' => false, 'message' => $validations->errors()]);}
        $style->title = $request->title;
        if($request->hasFile("icon")){ $style->icon = $this->uploadImage('icon','uploads/styles/icons',$request,''); }
        if($request->hasFile("image")){ $style->image = $this->uploadImage('image','uploads/styles/images',$request,''); }
        if($request->hasFile("left_frame")){ $style->left_frame = $this->uploadImage('left_frame','uploads/styles/leftFrames',$request,''); }
        
        
        if($style->save()){
            if($request->vertical_check && (isset($request->vertical_prices) && count($request->vertical_prices) > 0)){
                foreach($request->vertical_prices as $key=>$value){
                    $styleDetail = isset($request->vertical_ids[$key]) && $request->vertical_ids[$key]!='0'? StyleDetail::find($request->vertical_ids[$key]) : new StyleDetail();
                    $styleDetail->style_id = $style->id;
                    $styleDetail->orientation = $request->vertical_check;
                    $styleDetail->price = $request->vertical_prices[$key];
                    $styleDetail->size = $request->vertical_sizes[$key];
                    $styleDetail->price = $request->vertical_prices[$key];
                    if(isset($request->file("vertical_images")[$key]) && (isset($request->vertical_ids[$key]) && $request->vertical_ids[$key]!='0')) 
                    {
                        $styleDetail->image = $this->uploadFiles($request->file("vertical_images")[$key],"uploads/styles/details/images"); 
                    }elseif(isset($request->file("vertical_images")[$key]) && (isset($request->vertical_ids[$key]) && $request->vertical_ids[$key]=='0')){
                        $styleDetail->image = $this->uploadFiles($request->file("vertical_images")[$key],"uploads/styles/details/images");
                    }
                    $styleDetail->save();
                }
            }else{
                StyleDetail::where("style_id",$style->id)->where("orientation","vertical")->delete();
            }

            if($request->horizontal_check && (isset($request->horizontal_prices) && count($request->horizontal_prices) > 0)){
                foreach($request->horizontal_prices as $key=>$value){
                    $styleDetail = isset($request->horizontal_ids[$key]) && $request->horizontal_ids[$key]!='0'? StyleDetail::find($request->horizontal_ids[$key]) : new StyleDetail();
                    $styleDetail->style_id = $style->id;
                    $styleDetail->orientation = $request->horizontal_check;
                    $styleDetail->price = $request->horizontal_prices[$key];
                    $styleDetail->size = $request->horizontal_sizes[$key];
                    if(isset($request->file("horizontal_images")[$key]) && (isset($request->horizontal_ids[$key]) && $request->horizontal_ids[$key]!='0')) 
                    { 
                        $styleDetail->image = $this->uploadFiles($request->file("horizontal_images")[$key],"uploads/styles/details/images"); 
                    }elseif(isset($request->file("horizontal_images")[$key]) && (isset($request->horizontal_ids[$key]) && $request->horizontal_ids[$key]=='0')){
                        $styleDetail->image = $this->uploadFiles($request->file("horizontal_images")[$key],"uploads/styles/details/images");
                    }
                    $styleDetail->save();
                }
            }else{
                StyleDetail::where("style_id",$style->id)->where("orientation","horizontal")->delete();
            }

            StyleDetail::whereIn("id",$arrDiff)->delete();

            return response()->json(['success' => true, 'message' => 'Style has been updated successfully']);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        if(Style::where('id',$id)->delete()){
            return response()->json(['success' => true, 'message' =>'Style has been deleted successfully']);
        }
    }

    // get rules for create and update
    private function rules($request ,$validationscCheck=false,$update=false)
    {
        $rules = [
            "title"  => ['required'],
        ];

        if(!$update){
            $rules["icon"]  = ['required'];
            $rules["image"]  = ['required'];
            $rules["left_frame"]  = ['required'];
        }

        if($request->vertical_check && !$update){
            $rules["vertical_sizes"]  = ['required'];
            $rules["vertical_prices"]  = ['required'];
            $rules["vertical_images"]  = ['required'];
        }elseif($request->vertical_check && $update && $validationscCheck){
            $rules["vertical_sizes"]  = ['required'];
            $rules["vertical_prices"]  = ['required'];
            $rules["vertical_images"]  = ['required'];
        }

        if(($request->vertical_check && $request->vertical_check) && $update && $validationscCheck){
            $rules["vertical_sizes"]  = ['required'];
            $rules["vertical_prices"]  = ['required'];
            $rules["vertical_images"]  = ['required'];
        }

        if($request->horizontal_check && !$update){
            $rules["horizontal_sizes"]  = ['required'];
            $rules["horizontal_prices"]  = ['required'];
            $rules["horizontal_images"]  = ['required'];
        }elseif ($request->horizontal_check && $update && $validationscCheck) {
            $rules["horizontal_sizes"]  = ['required'];
            $rules["horizontal_prices"]  = ['required'];
            $rules["horizontal_images"]  = ['required'];
        }
        

        if($request->vertical_check){
            $rules["vertical_sizes.*"]  = ['required'];
            $rules["vertical_prices.*"]  = ['required'];
            $rules["vertical_images.*"]  = ['required'];
        }

        if($request->horizontal_check){
            $rules["horizontal_sizes.*"]  = ['required'];
            $rules["horizontal_prices.*"]  = ['required'];
            $rules["horizontal_images.*"]  = ['required'];
        }

        if($request->vertical_check && $request->horizontal_check){
            $rules["vertical_sizes.*"]  = ['required'];
            $rules["vertical_prices.*"]  = ['required'];
            $rules["vertical_images.*"]  = ['required'];
            $rules["horizontal_sizes.*"]  = ['required'];
            $rules["horizontal_prices.*"]  = ['required'];
            $rules["horizontal_images.*"]  = ['required'];
        }

        return $rules;
    }

    // error messages for validation
    private function messages($request)
    {
        $messages = [];
        foreach($request->all() as $key=>$value){
            if(is_array($value)){
                foreach ($value as $k => $v) {
                    $arr = explode('_',$key);
                    $first = ucfirst($arr[0]);
                    $second = str_replace('s','',$arr[1]);
                    $kkk = $k + 1;
                    $messages["$key.$k.required"] = " $first entry number $kkk $second field is required";
                }
            }
        }
        return $messages;
    }
}